/* GMM with mata */
clear
set obs 100
set seed 738595
gen x2 = rnormal()
gen x3 = rnormal()
gen z1 = rnormal()
gen z2 = rnormal()
gen z3 = rnormal()
gen IQ = rnormal()
gen x1 = z1 + z2 + z3 + IQ + rnormal()
gen y = 0.5 + 2*x1 + 2*x2 + 2*x3 + IQ + rnormal()
gen one = 1

global depvar "y"
global xvars "x1 x2 x3 one"
global zvars "x2 x3 z1 z2 z3 one"

mata
mata clear

y = NULL
X = NULL
Z = NULL 
st_view(y,.,"$depvar")
st_view(X,.,"$xvars")
st_view(Z,.,"$zvars")
/* Start with an identity matrix weighting matrix */
Wopt = I(cols(Z))
Wopt
/* perform GMM optimisation */
function oiv_gmm0(transmorphic M, real scalar todo,
 real rowvector b, q,S) {
 y = moptimize_util_depvar(M,1)
 xb = moptimize_util_xb(M,b,1)
 st_view(X,.,moptimize_util_userinfo(M, 1))
 st_view(Z,.,moptimize_util_userinfo(M, 2))
 e = y - xb
 q = Z'*e
}
M = moptimize_init()
moptimize_init_evaluator(M, & oiv_gmm0())
moptimize_init_evaluatortype(M, "q0")
moptimize_init_depvar(M, 1, "y")
moptimize_init_eq_indepvars(M, 1, "x1 x2 x3")
moptimize_init_userinfo(M, 1, ("x1", "x2", "x3","one"))
moptimize_init_userinfo(M, 2, ("x2", "x3","z1","z2","z3","one"))
moptimize_init_gnweightmatrix(M,Wopt)
moptimize_init_which(M, "min")
moptimize_init_technique(M, "gn")
moptimize(M)
/* get estimates */
b_gmm = moptimize_result_coefs(M)'
u_gmm = y - X*b_gmm
/* get estimate of weighting matrix */ 
Wopt = invsym(Z'*diag(u_gmm:^2)*Z)
/* re-optimize */
M = moptimize_init()
moptimize_init_evaluator(M, & oiv_gmm0())
moptimize_init_evaluatortype(M, "q0")
moptimize_init_depvar(M, 1, "y")
moptimize_init_eq_indepvars(M, 1, "x1 x2 x3")
moptimize_init_userinfo(M, 1, ("x1", "x2", "x3","one"))
moptimize_init_userinfo(M, 2, ("x2", "x3","z1","z2","z3","one"))
moptimize_init_gnweightmatrix(M,Wopt)
moptimize_init_which(M, "min")
moptimize_init_technique(M, "gn")
moptimize(M)
/* get estimes and calculate covar matrix */
b_gmm = moptimize_result_coefs(M)'
u_gmm = y - X*b_gmm
V_gmm = invsym((X'*Z)*invsym(Z'*diag(u_gmm:^2)*Z)*Z'X)
/* parse results to stata */
 moptimize_result_post(M, "robust")
st_matrix("b_gmm", b_gmm')
st_matrix("V_gmm", V_gmm)
end
/* post estimates to stata */
program drop _all
program mypost, eclass
ereturn repost b=b_gmm V=V_gmm 
end
mypost
/* display results */
eret di

/* GMM with ivregress */
ivregress gmm y (x1 = z1 z2 z3) x2 x3

/* GMM with gmm */

gmm (y - {b1}*x1 - {b2}*x2 - {b3}*x3 - {b0}), instruments(x2 x3 z1 z2 z3) wmatrix(robust)


