/* GMM with mata */
set more off
clear
set obs 100
set seed 6789
gen x2 = rnormal()
gen x3 = rnormal()
gen z1 = rnormal()
gen z2 = rnormal()
gen z3 = rnormal()
gen IQ = rnormal()
gen x1 = z1 + z2 + z3 + IQ + rnormal()
gen y1 = 0.5 + 2*x1 + 2*x2 + 2*x3 + IQ + rnormal()
gen y2 = 0.2 + 4*x1 + 4*x3 + IQ + rnormal()

gen one = 1

global depvar1 "y1"
global depvar2 "y2"
global xvars1 "x1 x2 x3 one"
global xvars2 "x1 x3 one"
global zvars1 "x2 x3 z1 z2 z3 one"
global zvars2 "x3 z1 z2 z3 one"

mata
mata clear

y1 = NULL
y2 = NULL
X1 = NULL
X2 = NULL
Z1 = NULL 
Z2 = NULL 
st_view(y1,.,"$depvar1")
st_view(y2,.,"$depvar2")
st_view(X1,.,"$xvars1")
st_view(X2,.,"$xvars2")
st_view(Z1,.,"$zvars1")
st_view(Z2,.,"$zvars2")
/* obtain number of individuals */
N = rows(y1)
/* Define y */
for (i=1;i<=N;i++) {
 if (i==1) y=(y1[i]\y2[i])
 else y = (y\(y1[i]\y2[i]))
}
/* Define X */
for (i=1;i<=N;i++) {
 if (i==1) X=blockdiag(X1[i,],X2[i,])
 else {
  X = (X\blockdiag(X1[i,],X2[i,]))
 }
}
/* Define Z */
for (i=1;i<=N;i++) {
 if (i==1) Z=blockdiag(Z1[i,],Z2[i,])
 else {
  Z = (Z\blockdiag(Z1[i,],Z2[i,]))
 }
}
/* Start with an identity matrix weighting matrix */
Wopt = I(cols(Z))
/* perform GMM optimisation */
function oiv_gmm0(transmorphic M, real scalar todo,
 real rowvector b, q,S) {
 y1 = moptimize_util_depvar(M,1)
 y2 = moptimize_util_depvar(M,2)
 xb1 = moptimize_util_xb(M,b,1)
 xb2 = moptimize_util_xb(M,b,2)
 st_view(Z1,.,moptimize_util_userinfo(M, 2))
 st_view(Z2,.,moptimize_util_userinfo(M, 4))
 N = rows(y1)
 /* define xb */
 for (i=1;i<=N;i++) {
  if (i==1) xb=(xb1[i]\xb2[i])
  else xb = (xb\(xb1[i]\xb2[i]))
 }
 /* Define y */
 for (i=1;i<=N;i++) {
  if (i==1) y=(y1[i]\y2[i])
  else y = (y\(y1[i]\y2[i]))
 }
/* Define Z */
for (i=1;i<=N;i++) {
 if (i==1) Z=blockdiag(Z1[i,],Z2[i,])
 else {
  Z = (Z\blockdiag(Z1[i,],Z2[i,]))
 }
}
 e = y - xb
 q = Z'*e
}
M = moptimize_init()
moptimize_init_evaluator(M, & oiv_gmm0())
moptimize_init_evaluatortype(M, "q0")
moptimize_init_ndepvars(M, 2)
moptimize_init_depvar(M, 1, "y1")
moptimize_init_depvar(M, 2, "y2")
moptimize_init_eq_indepvars(M, 1, "x1 x2 x3")
moptimize_init_eq_indepvars(M, 2, "x1 x3")
moptimize_init_userinfo(M, 1, ("x1", "x2", "x3","one"))
moptimize_init_userinfo(M, 2, ("x2", "x3","z1","z2","z3","one"))
moptimize_init_userinfo(M, 3, ("x1", "x3","one"))
moptimize_init_userinfo(M, 4, ("x3","z1","z2","z3","one"))
moptimize_init_gnweightmatrix(M,Wopt)
moptimize_init_which(M, "min")
moptimize_init_technique(M, "gn")
moptimize(M)
/* get estimates */
b_gmm = moptimize_result_coefs(M)'
u_gmm = y - X*b_gmm
/* get estimate of optimal W */
sel11 = J(N,1,1)                     
sel12 = J(N,1,0)
for (i=1;i<=N;i++) {
 if (i==1) sel1=(sel11[i]\sel12[i])
 else sel1 =(sel1\(sel11[i]\sel12[i]))   
}
u1_gmm=NULL
st_select(u1_gmm,u_gmm,sel1)              // get residual in eq 1
sel21 = J(N,1,0)
sel22 = J(N,1,1)
for (i=1;i<=N;i++) {
 if (i==1) sel2=(sel21[i]\sel22[i])
 else sel2 =(sel2\(sel21[i]\sel22[i]))
}
u2_gmm=NULL
st_select(u2_gmm,u_gmm,sel2)              // get residual in eq 1
for (i=1;i<=N;i++) {
 if (i==1) {
  Zi = blockdiag(Z1[i,],Z2[i,])
  ui = u1_gmm[i]\u2_gmm[i]
  D = Zi'*ui*ui'*Zi
 }                                       // get N^(-1) Sum(Zi'uiui'Zi)
 else {
  Zi = blockdiag(Z1[i,],Z2[i,])
  ui = u1_gmm[i]\u2_gmm[i]
   D = D + Zi'*ui*ui'*Zi
 }
}
D = (1/N)*D
Wopt = invsym(D)
/* re-optimize */
M = moptimize_init()
moptimize_init_evaluator(M, & oiv_gmm0())
moptimize_init_evaluatortype(M, "q0")
moptimize_init_ndepvars(M, 2)
moptimize_init_depvar(M, 1, "y1")
moptimize_init_depvar(M, 2, "y2")
moptimize_init_eq_indepvars(M, 1, "x1 x2 x3")
moptimize_init_eq_indepvars(M, 2, "x1 x3")
moptimize_init_userinfo(M, 1, ("x1", "x2", "x3","one"))
moptimize_init_userinfo(M, 2, ("x2", "x3","z1","z2","z3","one"))
moptimize_init_userinfo(M, 3, ("x1", "x3","one"))
moptimize_init_userinfo(M, 4, ("x3","z1","z2","z3","one"))
moptimize_init_gnweightmatrix(M,Wopt)
moptimize_init_which(M, "min")
moptimize_init_technique(M, "gn")
moptimize(M)
/* get estimes  */
b_gmm = moptimize_result_coefs(M)'
u_gmm = y - X*b_gmm
/* get estimate of D */
sel11 = J(N,1,1)                     
sel12 = J(N,1,0)
for (i=1;i<=N;i++) {
 if (i==1) sel1=(sel11[i]\sel12[i])
 else sel1 =(sel1\(sel11[i]\sel12[i]))   
}
u1_gmm=NULL
st_select(u1_gmm,u_gmm,sel1)              // get residual in eq 1
sel21 = J(N,1,0)
sel22 = J(N,1,1)
for (i=1;i<=N;i++) {
 if (i==1) sel2=(sel21[i]\sel22[i])
 else sel2 =(sel2\(sel21[i]\sel22[i]))
}
u2_gmm=NULL
st_select(u2_gmm,u_gmm,sel2)              // get residual in eq 1
for (i=1;i<=N;i++) {
 if (i==1) {
  Zi = blockdiag(Z1[i,],Z2[i,])
  ui = u1_gmm[i]\u2_gmm[i]
  D = Zi'*ui*ui'*Zi
 }                                       // get N^(-1) Sum(Zi'uiui'Zi)
 else {
  Zi = blockdiag(Z1[i,],Z2[i,])
  ui = u1_gmm[i]\u2_gmm[i]
   D = D + Zi'*ui*ui'*Zi
 }
}
//D = (1/N)*D
Wopt = invsym(D)
/* Calculate covariance matrix */
V_gmm = invsym((X'*Z)*Wopt*Z'X)
/* parse results to stata */
moptimize_result_post(M, "robust")
st_matrix("b_gmm", b_gmm')
st_matrix("V_gmm", V_gmm)
end
/* post estimates to stata */
program drop _all
program mypost, eclass
ereturn repost b=b_gmm V=V_gmm 
end
mypost
/* display results */
eret di

/* GMM with gmm */

#delimit ;
gmm (y1 - {b1}*x1 - {b2}*x2 - {b3}*x3 - {b0}) 
 (y2 - {g1}*x1 -  {g2}*x3 - {g0}), 
 instruments(1: x2 x3 z1 z2 z3) 
 instruments(2: x3 z1 z2 z3) 
 wmatrix(robust) winitial(identity);
#delimit cr

*/
